@@ -6,9 +6,9 @@
 ### Data structure for CONTINUOUS SPATIO-temporal infectious disease case data
 ### and a spatio-temporal grid of endemic covariates
 ###
-### Copyright (C) 2009-2014 Sebastian Meyer
-### $Revision: 885 $
-### $Date: 2014-04-05 21:19:32 +0200 (Sat, 05 Apr 2014) $
+### Copyright (C) 2009-2016 Sebastian Meyer
+### $Revision: 1750 $
+### $Date: 2016-06-06 20:29:40 +0200 (Mon, 06. Jun 2016) $
 ################################################################################
 
 
@@ -63,11 +63,12 @@ as.epidataCS <- function (events, stgrid, W, qmatrix = diag(nTypes),
     tiles <- NULL                       # FIXME: add argument to as.epidataCS
     stgrid <- if (missing(stgrid) && inherits(tiles, "SpatialPolygons")) {
         if (verbose) cat("\t(missing, using time-constant 'tiles' grid)\n")
-        stgrid_template <- data.frame(start = min(events$time),
-                                      stop = max(events$time),
-                                      tile = row.names(tiles),
-                                      area = sapply(tiles@polygons, slot, "area"),
-                                      check.rows = FALSE, check.names = FALSE)
+        stgrid_template <- data.frame(
+            start = min(events$time),
+            stop = max(events$time),
+            tile = row.names(tiles),
+            area = areaSpatialPolygons(tiles, byid = TRUE),
+            check.rows = FALSE, check.names = FALSE)
         check_stgrid(stgrid_template, verbose = FALSE)
     } else {
         check_stgrid(stgrid, T, verbose = verbose)
@@ -108,14 +109,13 @@ as.epidataCS <- function (events, stgrid, W, qmatrix = diag(nTypes),
     }
 
     # Some basic quantities
-    eventCoords <- coordinates(events)
-    nEvents <- nrow(eventCoords)
+    nEvents <- length(events)
     timeRange <- with(stgrid, c(start[1], stop[length(stop)]))
 
     # Are event times covered by stgrid?
     if (verbose) cat("Checking if all events are covered by 'stgrid' ...\n")
     ## FIXME: what about pre-history events? don't need stgrid-data for them
-    if (events$time[1] <= timeRange[1] || events$time[nEvents] > timeRange[2]) {
+    if (events$time[1L] <= timeRange[1L] || events$time[nEvents] > timeRange[2L]) {
         stop("event times are not covered by 'stgrid': must be in (",
              timeRange[1L],",",timeRange[2L],"]")
     }
@@ -128,21 +128,10 @@ as.epidataCS <- function (events, stgrid, W, qmatrix = diag(nTypes),
     }
     events$tile <- .events.tile
 
-    # Calculate time point of removal, when event is definitely no longer infective
-    removalTimes <- events$time + events$eps.t
-
-    # Calculate distance matrix of events
-    if (verbose) cat("Calculating euclidean distance matrix of events ...\n")
-    eventDists <- as.matrix(dist(eventCoords, method = "euclidean"))
-    #diag(eventDists) <- Inf   # infinite distance to oneself (no self-infection), not needed
-
     # Map events to corresponding grid cells
-    # Also precalculate possible origins of events (other infected individuals)
-    if (verbose) cat("Mapping events to 'stgrid' cells and",
-                     "determining potential event sources ...\n")
+    if (verbose) cat("Mapping events to 'stgrid' cells ...\n")
     withPB <- verbose && interactive()
     gridcellsOfEvents <- integer(nEvents)
-    eventSources <- vector(nEvents, mode = "list")
     if (withPB) pb <- txtProgressBar(min=0, max=nEvents, initial=0, style=3)
     for (i in seq_len(nEvents)) {
         idx <- gridcellOfEvent(events$time[i], events$tile[i], stgrid)
@@ -151,9 +140,6 @@ as.epidataCS <- function (events, stgrid, W, qmatrix = diag(nTypes),
                  " and tile \"", events$tile[i], "\" in 'stgrid'")
         }
         gridcellsOfEvents[i] <- idx
-        eventSources[[i]] <- determineSources(
-            i, events$time, removalTimes, eventDists[i,], events$eps.s, events$type, qmatrix
-        )
         if (withPB) setTxtProgressBar(pb, i)
     }
     if (withPB) close(pb)
@@ -176,13 +162,17 @@ as.epidataCS <- function (events, stgrid, W, qmatrix = diag(nTypes),
     # Calculate observed infection length = min(T-time, eps.t) for use in log-likelihood
     events$.obsInfLength <- with(events@data, pmin(timeRange[2]-time, eps.t))
 
-    # Attach possible eventSources (infective individuals) to events
-    events$.sources <- eventSources
+    # Determine potential source events (infective individuals) of each event
+    if (verbose) cat("Determining potential event sources ...\n")
+    events$.sources <- determineSources(
+        eventTimes = events$time, eps.t = events$eps.t,
+        eventCoords = coordinates(events), eps.s = events$eps.s,
+        eventTypes = events$type, qmatrix = qmatrix)
 
     # Calculate minimal distance of event locations from the polygonal boundary
     if (verbose) cat("Calculating the events' distances to the boundary ...\n")
     Wowin <- as(W, "owin")              # imported from polyCub
-    events$.bdist <- bdist(eventCoords, Wowin)
+    events$.bdist <- bdist(coordinates(events), Wowin)
 
     # Construct spatial influence regions around events
     if (verbose) cat("Constructing spatial influence regions around events ...\n")
@@ -217,6 +207,12 @@ check_events <- function (events, dropTypes = TRUE, verbose = TRUE)
         stop("only two spatial dimensions are supported")
     }
 
+    # check suitability of Euclidean geometry
+    if (identical(FALSE, is.projected(events))) { # is.projected may return NA
+        warning("\"epidataCS\" expects planar coordinates; ",
+                "see 'spTransform' in package \"rgdal\"")
+    }
+
     # Check existence of type column
     if (verbose) cat("\tChecking 'type' column ... ")
     events$type <- if ("type" %in% names(events)) {
@@ -270,9 +266,10 @@ check_events <- function (events, dropTypes = TRUE, verbose = TRUE)
     events <- events[order(events$time),]
     
     # First obligatory columns then remainders (epidemic covariates)
-    obligColsIdx <- match(obligColsNames_events, names(events))
+    obligColsIdx <- match(obligColsNames_events, names(events@data))
     covarColsIdx <- setdiff(seq_along(events@data), obligColsIdx)
-    events <- events[c(obligColsIdx, covarColsIdx)]
+    events@data <- events@data[c(obligColsIdx, covarColsIdx)]
+    events@coords.nrs <- numeric(0L)  # forget index of coordinate columns
 
     # Done.
     return(events)
@@ -399,8 +396,9 @@ check_W <- function (W, area.other = NULL, other, tolerance = 0.001)
 
 check_W_area <- function (W, area.other, other, tolerance = 0.001)
 {
-    area.W <- sum(sapply(W@polygons, slot, "area"))
-    if (!isTRUE(all.equal(area.other, area.W, tolerance = tolerance)))
+    area.W <- areaSpatialPolygons(W)
+    if (!isTRUE(all.equal.numeric(area.other, area.W, tolerance = tolerance,
+                                  check.attributes = FALSE)))
         warning("area of 'W' (", area.W, ") differs from ",
                 "total tile area in '", other, "' (", area.other, ")")
 }
@@ -434,7 +432,7 @@ check_tiles <- function (tiles, levels,
     }
 
     ## check areas
-    areas.tiles <- sapply(tiles[levels,]@polygons, slot, "area")
+    areas.tiles <- areaSpatialPolygons(tiles[levels,], byid = TRUE)
     if (!is.null(areas.stgrid)) {
         check_tiles_areas(areas.tiles, areas.stgrid, tolerance=tolerance)
     }
@@ -476,9 +474,9 @@ check_tiles_events <- function (tiles, events)
 
 check_tiles_areas <- function (areas.tiles, areas.stgrid, tolerance = 0.05)
 {
-    stopifnot(length(areas.tiles) == length(areas.stgrid))
-    areas_all_equal <- all.equal(areas.stgrid, areas.tiles,
-                                 tolerance = tolerance)
+    areas_all_equal <- all.equal.numeric(areas.stgrid, areas.tiles,
+                                         tolerance = tolerance,
+                                         check.attributes = FALSE)
     if (!isTRUE(areas_all_equal))
         warning("tile areas in 'stgrid' differ from areas of 'tiles': ",
                 areas_all_equal)
@@ -529,5 +527,6 @@ check_tiles_areas <- function (areas.tiles, areas.stgrid, tolerance = 0.05)
         SIMPLIFY = FALSE, USE.NAMES = FALSE)
     
     attr(res, "nCircle2Poly") <- npoly
+    attr(res, "clipper") <- clipper
     res
 }
